@@ -1,7 +1,7 @@
-# Copyright 1999-2009 Gentoo Foundation
+# Copyright 1999-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: BinpkgFetcher.py 15137 2009-12-24 04:01:10Z zmedico $
 
+from _emerge.AsynchronousLock import AsynchronousLock
 from _emerge.SpawnProcess import SpawnProcess
 try:
 	from urllib.parse import urlparse as urllib_parse_urlparse
@@ -11,6 +11,7 @@ import stat
 import sys
 import portage
 from portage import os
+from portage.util._pty import _create_pty_or_pipe
 
 if sys.hexversion >= 0x3000000:
 	long = int
@@ -27,9 +28,6 @@ class BinpkgFetcher(SpawnProcess):
 
 	def _start(self):
 
-		if self.cancelled:
-			return
-
 		pkg = self.pkg
 		pretend = self.pretend
 		bintree = pkg.root_config.trees["bintree"]
@@ -56,15 +54,15 @@ class BinpkgFetcher(SpawnProcess):
 			rel_uri = bintree._remotepkgs[pkg.cpv].get("PATH")
 			if not rel_uri:
 				rel_uri = pkg.cpv + ".tbz2"
-			uri = bintree._remote_base_uri.rstrip("/") + \
-				"/" + rel_uri.lstrip("/")
+			remote_base_uri = bintree._remotepkgs[pkg.cpv]["BASE_URI"]
+			uri = remote_base_uri.rstrip("/") + "/" + rel_uri.lstrip("/")
 		else:
 			uri = settings["PORTAGE_BINHOST"].rstrip("/") + \
 				"/" + pkg.pf + ".tbz2"
 
 		if pretend:
 			portage.writemsg_stdout("\n%s\n" % uri, noiselevel=-1)
-			self.returncode = os.EX_OK
+			self._set_returncode((self.pid, os.EX_OK << 8))
 			self.wait()
 			return
 
@@ -93,17 +91,33 @@ class BinpkgFetcher(SpawnProcess):
 		# Redirect all output to stdout since some fetchers like
 		# wget pollute stderr (if portage detects a problem then it
 		# can send it's own message to stderr).
-		fd_pipes.setdefault(0, sys.stdin.fileno())
-		fd_pipes.setdefault(1, sys.stdout.fileno())
-		fd_pipes.setdefault(2, sys.stdout.fileno())
+		fd_pipes.setdefault(0, portage._get_stdin().fileno())
+		fd_pipes.setdefault(1, sys.__stdout__.fileno())
+		fd_pipes.setdefault(2, sys.__stdout__.fileno())
 
 		self.args = fetch_args
 		self.env = fetch_env
+		if settings.selinux_enabled():
+			self._selinux_type = settings["PORTAGE_FETCH_T"]
 		SpawnProcess._start(self)
 
+	def _pipe(self, fd_pipes):
+		"""When appropriate, use a pty so that fetcher progress bars,
+		like wget has, will work properly."""
+		if self.background or not sys.__stdout__.isatty():
+			# When the output only goes to a log file,
+			# there's no point in creating a pty.
+			return os.pipe()
+		stdout_pipe = None
+		if not self.background:
+			stdout_pipe = fd_pipes.get(1)
+		got_pty, master_fd, slave_fd = \
+			_create_pty_or_pipe(copy_term_size=stdout_pipe)
+		return (master_fd, slave_fd)
+
 	def _set_returncode(self, wait_retval):
 		SpawnProcess._set_returncode(self, wait_retval)
-		if self.returncode == os.EX_OK:
+		if not self.pretend and self.returncode == os.EX_OK:
 			# If possible, update the mtime to match the remote package if
 			# the fetcher didn't already do it automatically.
 			bintree = self.pkg.root_config.trees["bintree"]
@@ -140,8 +154,16 @@ class BinpkgFetcher(SpawnProcess):
 		if self._lock_obj is not None:
 			raise self.AlreadyLocked((self._lock_obj,))
 
-		self._lock_obj = portage.locks.lockfile(
-			self.pkg_path, wantnewlockfile=1)
+		async_lock = AsynchronousLock(path=self.pkg_path,
+			scheduler=self.scheduler)
+		async_lock.start()
+
+		if async_lock.wait() != os.EX_OK:
+			# TODO: Use CompositeTask for better handling, like in EbuildPhase.
+			raise AssertionError("AsynchronousLock failed with returncode %s" \
+				% (async_lock.returncode,))
+
+		self._lock_obj = async_lock
 		self.locked = True
 
 	class AlreadyLocked(portage.exception.PortageException):
@@ -150,7 +172,7 @@ class BinpkgFetcher(SpawnProcess):
 	def unlock(self):
 		if self._lock_obj is None:
 			return
-		portage.locks.unlockfile(self._lock_obj)
+		self._lock_obj.unlock()
 		self._lock_obj = None
 		self.locked = False
 
