@@ -1,4 +1,4 @@
-# Copyright 1999-2011 Gentoo Foundation
+# Copyright 1999-2012 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from _emerge.EbuildExecuter import EbuildExecuter
@@ -30,12 +30,13 @@ class EbuildBuild(CompositeTask):
 		pkg = self.pkg
 		settings = self.settings
 
-		rval = _check_temp_dir(settings)
-		if rval != os.EX_OK:
-			self.returncode = rval
-			self._current_task = None
-			self.wait()
-			return
+		if not self.opts.fetchonly:
+			rval = _check_temp_dir(settings)
+			if rval != os.EX_OK:
+				self.returncode = rval
+				self._current_task = None
+				self.wait()
+				return
 
 		root_config = pkg.root_config
 		tree = "porttree"
@@ -47,7 +48,7 @@ class EbuildBuild(CompositeTask):
 			settings.configdict["pkg"]["MERGE_TYPE"] = "buildonly"
 		else:
 			settings.configdict["pkg"]["MERGE_TYPE"] = "source"
-		ebuild_path = portdb.findname(pkg.cpv)
+		ebuild_path = portdb.findname(pkg.cpv, myrepo=pkg.repo)
 		if ebuild_path is None:
 			raise AssertionError("ebuild not found for '%s'" % pkg.cpv)
 		self._ebuild_path = ebuild_path
@@ -65,9 +66,8 @@ class EbuildBuild(CompositeTask):
 		prefetcher = self.prefetcher
 		if prefetcher is None:
 			pass
-		elif not prefetcher.isAlive():
-			prefetcher.cancel()
-		elif prefetcher.poll() is None:
+		elif prefetcher.isAlive() and \
+			prefetcher.poll() is None:
 
 			waiting_msg = "Fetching files " + \
 				"in the background. " + \
@@ -107,6 +107,10 @@ class EbuildBuild(CompositeTask):
 
 	def _prefetch_exit(self, prefetcher):
 
+		if self._was_cancelled():
+			self.wait()
+			return
+
 		opts = self.opts
 		pkg = self.pkg
 		settings = self.settings
@@ -124,6 +128,7 @@ class EbuildBuild(CompositeTask):
 			else:
 				fetcher = EbuildFetcher(
 					config_pool=self.config_pool,
+					ebuild_path=self._ebuild_path,
 					fetchall=self.opts.fetch_all_uri,
 					fetchonly=self.opts.fetchonly,
 					background=False,
@@ -167,17 +172,44 @@ class EbuildBuild(CompositeTask):
 		portage.prepare_build_dirs(self.pkg.root, self.settings, 1)
 
 		fetcher = EbuildFetcher(config_pool=self.config_pool,
+			ebuild_path=self._ebuild_path,
 			fetchall=self.opts.fetch_all_uri,
 			fetchonly=self.opts.fetchonly,
 			background=self.background,
 			logfile=self.settings.get('PORTAGE_LOG_FILE'),
 			pkg=self.pkg, scheduler=self.scheduler)
 
-		self._start_task(fetcher, self._fetch_exit)
+		try:
+			already_fetched = fetcher.already_fetched(self.settings)
+		except portage.exception.InvalidDependString as e:
+			msg_lines = []
+			msg = "Fetch failed for '%s' due to invalid SRC_URI: %s" % \
+				(self.pkg.cpv, e)
+			msg_lines.append(msg)
+			fetcher._eerror(msg_lines)
+			portage.elog.elog_process(self.pkg.cpv, self.settings)
+			self.returncode = 1
+			self._current_task = None
+			self._unlock_builddir()
+			self.wait()
+			return
+
+		if already_fetched:
+			# This case is optimized to skip the fetch queue.
+			fetcher = None
+			self._fetch_exit(fetcher)
+			return
+
+		# Allow the Scheduler's fetch queue to control the
+		# number of concurrent fetchers.
+		fetcher.addExitListener(self._fetch_exit)
+		self._task_queued(fetcher)
+		self.scheduler.fetch.schedule(fetcher)
 
 	def _fetch_exit(self, fetcher):
 
-		if self._default_exit(fetcher) != os.EX_OK:
+		if fetcher is not None and \
+			self._default_exit(fetcher) != os.EX_OK:
 			self._fetch_failed()
 			return
 
@@ -196,9 +228,11 @@ class EbuildBuild(CompositeTask):
 		#buildsyspkg: Check if we need to _force_ binary package creation
 		self._issyspkg = "buildsyspkg" in features and \
 				system_set.findAtomForPackage(pkg) and \
-				not opts.buildpkg
+				"buildpkg" not in features and \
+				opts.buildpkg != 'n'
 
-		if opts.buildpkg or self._issyspkg:
+		if ("buildpkg" in features or self._issyspkg) \
+			and not self.opts.buildpkg_exclude.findAtomForPackage(pkg):
 
 			self._buildpkg = True
 
@@ -226,8 +260,8 @@ class EbuildBuild(CompositeTask):
 		# to be displayed for problematic packages even though they do
 		# not set RESTRICT=fetch (bug #336499).
 
-		if 'fetch' not in self.pkg.metadata.restrict and \
-			'nofetch' not in self.pkg.metadata.defined_phases:
+		if 'fetch' not in self.pkg.restrict and \
+			'nofetch' not in self.pkg.defined_phases:
 			self._unlock_builddir()
 			self.wait()
 			return
@@ -315,7 +349,7 @@ class EbuildBuild(CompositeTask):
 			self._unlock_builddir()
 		self.wait()
 
-	def install(self):
+	def create_install_task(self):
 		"""
 		Install the package and then clean up and release locks.
 		Only call this after the build has completed successfully
@@ -331,7 +365,8 @@ class EbuildBuild(CompositeTask):
 		ebuild_path = self._ebuild_path
 		tree = self._tree
 
-		merge = EbuildMerge(find_blockers=self.find_blockers,
+		task = EbuildMerge(exit_hook=self._install_exit,
+			find_blockers=self.find_blockers,
 			ldpath_mtimes=ldpath_mtimes, logger=logger, pkg=pkg,
 			pkg_count=pkg_count, pkg_path=ebuild_path,
 			scheduler=self.scheduler,
@@ -344,10 +379,7 @@ class EbuildBuild(CompositeTask):
 			(pkg_count.curval, pkg_count.maxval, pkg.cpv)
 		logger.log(msg, short_msg=short_msg)
 
-		try:
-			rval = merge.execute()
-		finally:
-			self._unlock_builddir()
-
-		return rval
+		return task
 
+	def _install_exit(self, task):
+		self._unlock_builddir()
