@@ -44,7 +44,6 @@ options=[
 "--tree",
 "--unordered-display",
 "--update",
-"--verbose",
 "--verbose-main-repo-display",
 ]
 
@@ -65,7 +64,7 @@ shortmapping={
 "s":"--search",    "S":"--searchdesc",
 "t":"--tree",
 "u":"--update",
-"v":"--verbose",   "V":"--version"
+"V":"--version"
 }
 
 COWSAY_MOO = """
@@ -139,6 +138,7 @@ def insert_optional_args(args):
 		'--package-moves'        : y_or_n,
 		'--quiet'                : y_or_n,
 		'--quiet-build'          : y_or_n,
+		'--quiet-fail'           : y_or_n,
 		'--rebuild-if-new-slot': y_or_n,
 		'--rebuild-if-new-rev'   : y_or_n,
 		'--rebuild-if-new-ver'   : y_or_n,
@@ -150,6 +150,7 @@ def insert_optional_args(args):
 		"--use-ebuild-visibility": y_or_n,
 		'--usepkg'               : y_or_n,
 		'--usepkgonly'           : y_or_n,
+		'--verbose'              : y_or_n,
 	}
 
 	short_arg_opts = {
@@ -167,6 +168,7 @@ def insert_optional_args(args):
 		'k' : y_or_n,
 		'K' : y_or_n,
 		'q' : y_or_n,
+		'v' : y_or_n,
 	}
 
 	arg_stack = args[:]
@@ -541,6 +543,12 @@ def parse_opts(tmpcmdline, silent=False):
 			"choices"  : true_y_or_n,
 		},
 
+		"--quiet-fail": {
+			"help"     : "suppresses display of the build log on stdout",
+			"type"     : "choice",
+			"choices"  : true_y_or_n,
+		},
+
 		"--rebuild-if-new-slot": {
 			"help"     : ("Automatically rebuild or reinstall packages when slot/sub-slot := "
 				"operator dependencies can be satisfied by a newer slot, so that "
@@ -638,6 +646,13 @@ def parse_opts(tmpcmdline, silent=False):
 			"type"     : "choice",
 			"choices"  : true_y_or_n
 		},
+
+		"--verbose": {
+			"shortopt" : "-v",
+			"help"     : "verbose output",
+			"type"     : "choice",
+			"choices"  : true_y_or_n
+		},
 	}
 
 	from optparse import OptionParser
@@ -782,6 +797,9 @@ def parse_opts(tmpcmdline, silent=False):
 	if myoptions.quiet_build in true_y:
 		myoptions.quiet_build = 'y'
 
+	if myoptions.quiet_fail in true_y:
+		myoptions.quiet_fail = 'y'
+
 	if myoptions.rebuild_if_new_slot in true_y:
 		myoptions.rebuild_if_new_slot = 'y'
 
@@ -917,6 +935,11 @@ def parse_opts(tmpcmdline, silent=False):
 	else:
 		myoptions.usepkgonly = None
 
+	if myoptions.verbose in true_y:
+		myoptions.verbose = True
+	else:
+		myoptions.verbose = None
+
 	for myopt in options:
 		v = getattr(myoptions, myopt.lstrip("--").replace("-", "_"))
 		if v:
